# Copyright (c) ModelScope Contributors. All rights reserved.
from dataclasses import dataclass
from typing import Any, Dict


@dataclass
class Response:
    """
    Represents a response generated by the agentic search system.
    """

    # The main content of the response
    content: str

    # Additional data or metadata associated with the response
    metadata: Dict[str, Any] = None

    def __post_init__(self):
        if self.metadata is None:
            self.metadata = {}
